@@ -14,11 +14,11 @@ Games::Cards::ShuffleTrack - Track cards through shuffles and cuts
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 =cut
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 my $cut_limits = {
 	normal  => [0.19, 0.82], # on a 52 cards deck, cut between 10 and 43 cards
@@ -46,10 +46,31 @@ my $shortcuts = {
     'bottom'  => -1,
 };
 
+my $expressions = {
+	A   => qr/A[CHSD]/,
+	2   => qr/2[CHSD]/,
+	3   => qr/3[CHSD]/,
+	4   => qr/4[CHSD]/,
+	5   => qr/5[CHSD]/,
+	6   => qr/6[CHSD]/,
+	7   => qr/7[CHSD]/,
+	8   => qr/8[CHSD]/,
+	9   => qr/9[CHSD]/,
+	10  => qr/10[CHSD]/,
+	J   => qr/J[CHSD]/,
+	Q   => qr/Q[CHSD]/,
+	K   => qr/K[CHSD]/,
+
+	C   => qr/(?:[A23456789JQK]|10)C/,
+	H   => qr/(?:[A23456789JQK]|10)H/,
+	S   => qr/(?:[A23456789JQK]|10)S/,
+	D   => qr/(?:[A23456789JQK]|10)D/,
+};
+
 
 =head1 SYNOPSIS
 
-This module allows you to simulate true and false shuffles and cuts.
+This module allows you to simulate shuffles and cuts.
 
 	use Games::Cards::ShuffleTrack;
 
@@ -98,6 +119,10 @@ As an example, some card representations:
 	4D - 4 of Diamonds
 	KH - King of Hearts
 
+Still, you can add whichever cards you want to the deck:
+
+	$deck->put( 'Joker' ); # place a Joker on top of the deck
+
 
 =head1 SUBROUTINES/METHODS
 
@@ -134,15 +159,28 @@ You can also specify the starting order of the deck among the following:
 
 	my $deck = Games::Cards::ShuffleTrack->new( 'fournier' );
 
+You can also set your own order:
+
+	my $pile = Games::Cards::ShufleTrack->new( [qw/10S JS QS KS AS/] );
+
 =cut
 
+# TODO: What happens if the user has a typo in the order?
 sub new {
 	my ($self, $order) = @_;
 	   $order ||= 'new_deck_order';
-	my $deck    = $decks->{ $order };
+
+	my $cards;
+	if ( ref $order eq 'ARRAY' ) {
+		$cards = $order;
+	}
+	else {
+		$cards = $decks->{ $order };
+	}
+
 	return bless {
-		'deck'        => $deck,
-		'original'    => $deck,
+		'deck'        => $cards,
+		'original'    => $cards,
 		'orientation' => 'down',
 	}, $self;
 }
@@ -163,7 +201,7 @@ sub restart {
 }
 
 
-=head3 deck_size
+=head3 size, deck_size
 
 Returns the size of the deck.
 
@@ -171,12 +209,33 @@ Returns the size of the deck.
 
 =cut
 
+sub size {
+	my $self = shift;
+	return $self->deck_size;
+}
+
 sub deck_size {
 	my $self = shift;
 	return scalar @{$self->{'deck'}};
 }
 
 
+=head3 original_size
+
+Returns the original size of the deck.
+
+	if ($size < 0.5 * $deck->original_size) {
+		# if the deck has been exausted by over 50%
+	}
+
+=cut
+
+sub original_size {
+	my $self = shift;
+	return scalar @{$self->{'original'}};
+}
+
+
 =head3 get_deck
 
 Returns the deck (a reference to a list of strings).
@@ -185,7 +244,9 @@ Returns the deck (a reference to a list of strings).
 
 =cut
 
-sub get_deck { # TODO: use wantarray to allow for an array to be returned
+# TODO: should we return a copy of the list instead of the list itself?
+# TODO: use wantarray to allow for an array to be returned?
+sub get_deck {
 	my $self = shift;
 	return $self->{'deck'};
 }
@@ -225,9 +286,100 @@ sub turn {
 
 	$self->{'orientation'} = $self->orientation eq 'down' ? 'up' : 'down';
 
-	$self->_set_deck( reverse @{$self->get_deck} );
+	return $self->_set_deck( reverse @{$self->get_deck} );
+}
 
-	return $self;
+=head3 count
+
+Counts how many cards with specific characteristics are in the deck.
+
+	# how many tens
+	$deck->count( '10' );
+
+	# how many Clubs
+	$deck->count( 'C' );
+
+	# how many Clubs and Hearts
+	my ($clubs, $hearts) = $deck->count( 'C', 'H' );
+	my $clubs_and_hearts = $deck->count( 'C', 'H' );
+
+	# how many Jokers
+	$deck->count( 'Joker' );
+
+Since you can add whichever card you want to the deck, it should be noted how searching for values and suits works:
+
+=over 4
+
+=item * If looking for a value from 2 to A, you'll get the amount of cards with that value and one of the four suits
+
+=item * If looking for a suit (C, H, S, D), you'll get the amount of cards with a value from 2 through Ace and that suit
+
+=item * If looking for anything else, that something is compared to the whole card
+
+=back
+
+It is important to note:
+
+	my $total = $deck->count( 'JC' );   # holds 4
+	my $total = $deck->count( 'C', J ); # holds 16, because the JC is only counted once
+	my @total = $deck->count( 'C', J ); # holds (13, 4)
+
+Also:
+
+	$deck->put( 'Joker' );
+	$deck->put( 'Signed Joker' );
+	$deck->count( 'Joker' ); # returns 2
+
+	$deck->put( 'Signed 4C' );
+	$deck->count( '4C' ); # returns 2, because you didn't removed the previous one
+
+=cut
+
+sub count {
+	my $self = shift;
+	my @results;
+
+	if ( wantarray and @_ > 1 ) {
+		return $self->_count_each( @_ );
+	}
+	else {
+		return $self->_count_all( @_ );
+	}
+}
+
+sub _count_each {
+	my $self = shift;
+	my @results;
+	while (my $param = shift) {
+		if ( exists $expressions->{$param} ) {
+			push @results, scalar grep { /$expressions->{$param}/ } @{$self->get_deck};
+		}
+		else {
+			push @results, scalar grep { /$param/ } @{$self->get_deck};
+		}
+	}
+	return @results;
+}
+
+sub _count_all {
+	my $self = shift;
+
+	my @expressions;
+	for my $param (@_) {
+		push @expressions, exists $expressions->{$param} ?
+							$expressions->{$param} :
+							qr/$param/x,
+	}
+
+	my @results;
+	for my $card (@{$self->get_deck}) {
+
+		if (any { $card =~ $_ } @expressions) {
+			push @results, $card;
+		}
+
+	}
+	return scalar @results;
 }
 
 
@@ -249,7 +401,13 @@ You can specify how many times you want to go through the deck (which is basical
 
 sub overhand_shuffle {
 	my $self  = shift;
-	my $times = shift || 1;
+	my $times = shift;
+
+	if (not defined $times) {
+		$times = 1;
+	}
+
+	return $self if $times < 1;
 
 	$self->_packet_transfer( 1, 10 );
 
@@ -258,6 +416,65 @@ sub overhand_shuffle {
 		   $self
 }
 
+
+=head4 run
+
+The act of running cards is similar to the overhand shuffle, but instead of in packets the cards are run singly.
+
+	$deck->run( 10 );
+
+When running cards you can choose whether to drop those cards on the top or on the bottom of the deck. By default, the cards are moved to the bottom of the deck.
+
+	$deck->run( 10, 'drop-top' );
+	$deck->run( 10, 'drop-bottom' );
+
+Running cards basically reverses their order.
+
+If no number is given then no cards are run.
+
+If we're doing multiple runs we can set everything at the same time:
+
+	$deck->run( 4, 6, 2 );
+
+=cut
+
+# TODO: review this code
+sub run {
+	my $self = shift;
+
+	my @number_of_cards;
+	my $where_to_drop;
+
+	while ( my $param = shift ) {
+		if ( looks_like_number( $param ) ) {
+			push @number_of_cards, $param;
+		}
+		else {
+			$where_to_drop = $param;
+		}
+	}
+
+	@number_of_cards || return $self;
+	$where_to_drop   ||= 'drop-bottom';
+
+	my $number_of_cards = shift @number_of_cards;
+	$number_of_cards > 0 or return $self;
+
+	# take cards from top and reverse their order
+	my @deck = @{$self->get_deck};
+	my @run  = reverse splice @deck, 0, $number_of_cards;
+
+	if ( $where_to_drop eq 'drop-top' ) {
+		$self->_set_deck( @run, @deck );
+	}
+	else { # drop-bottom is the default
+		$self->_set_deck( @deck, @run );
+	}
+
+	return $self->run( @number_of_cards, $where_to_drop );
+}
+
+
 =head3 Hindu Shuffle
 
 =head4 hindu_shuffle
@@ -304,46 +521,6 @@ sub _packet_transfer {
 }
 
 
-=head4 run
-
-The act of running cards is similar to the overhand shuffle, but instead of in packets the cards are run singly.
-
-	$deck->run( 10 );
-
-When running cards you can choose whether to drop those cards on the top or on the bottom of the deck. By default, the cards are moved to the bottom of the deck.
-
-	$deck->run( 10, 'drop-top' );
-	$deck->run( 10, 'drop-bottom' );
-
-Running cards basically reverses their order.
-
-If no number is given then no cards are run.
-
-=cut
-
-sub run {
-	my $self            = shift;
-	my $number_of_cards = shift || return $self;
-	my $where_to_drop   = shift || 'drop-bottom';
-
-	$number_of_cards > 0 or return $self;
-
-
-	# take cards from top and reverse their order
-	my @deck = @{$self->get_deck};
-	my @run  = reverse splice @deck, 0, $number_of_cards;
-
-	if ( $where_to_drop eq 'drop-top' ) {
-		$self->_set_deck( @run, @deck );
-	}
-	else { # drop-bottom is the default
-		$self->_set_deck( @deck, @run );
-	}
-
-	return $self;
-}
-
-
 =head3 Riffle Shuffle
 
 =head4 riffle_shuffle
@@ -366,6 +543,8 @@ You can also decide where to cut the deck for the shuffle:
 # TODO: add an option for an out-shuffle
 # TODO: add an option to control top or bottom stock
 # TODO: when dropping cards, should we favor numbers 2 and 3?
+# TODO: with a lot of cards, the riffle should be done breaking the deck in piles
+# TODO: consider how fast each half is being depleted and whether the packets riffled on each side are of similar sizes
 sub riffle_shuffle {
 	my $self  = shift;
 	my $depth = shift;
@@ -548,8 +727,9 @@ This method returns the new pile.
 
 =cut
 
+# FIXME: in some situations this method alters the original order of the deck
 sub cut_to {
-	my $self        = shift;
+	my $self = shift;
 
 	# create the new pile if required
 	my $new_pile;
@@ -560,6 +740,7 @@ sub cut_to {
 		$new_pile = Games::Cards::ShuffleTrack->new( 'empty' );
 	}
 
+	# TODO: could this be done with _cut_depth? (perhaps changing it a bit)
 	# set the position
 	my $lower_limit = shift;
 	my $upper_limit = shift;
@@ -591,6 +772,7 @@ Places a pile of cards on top of the deck.
 
 =cut
 
+# TODO place_on_top and put share similar code; review
 sub place_on_top {
 	my $self = shift;
 	my @pile = @_;
@@ -652,6 +834,33 @@ sub running_cuts {
 }
 
 
+=head3 bury
+
+Buries a group of cards under another group:
+
+	# bury the top 10 cards under the following 3 cards
+	$deck->bury(10, 3);
+
+	# move the top card to the 13th position
+	$deck->bury( 1, 12 );
+
+=cut
+
+sub bury {
+	my $self          = shift;
+	my $first_amount  = shift;
+	my $second_amount = shift;
+
+	my @deck = @{$self->get_deck};
+
+	splice @deck, $second_amount, 0, splice @deck, 0, $first_amount;
+
+	$self->_set_deck( @deck );
+
+	return $self;
+}
+
+
 =head2 Handling cards
 
 There are a few different methods to track down cards.
@@ -828,9 +1037,16 @@ Inserts a card in a specified position in the deck. If the position isn't specif
 
 If the position doesn't exist the card will be replaced at the bottom of the deck.
 
+You can also add cards to negative positions, meaning that the resulting position will be that negative position:
+
+    # insert card so that it ends up being the last one in the deck
+	$deck->insert( $card, -1 );
+
+    # insert card so that it ends up being the 10th from the bottom
+	$deck->insert( $card, -10 );
+
 =cut
 
-# TODO: inserting in negative position?
 # TODO: what if the user inserts at position 0?
 sub insert {
 	my $self     = shift;
@@ -843,6 +1059,9 @@ sub insert {
 	elsif ( $position > $self->deck_size ) {
 		$position = $self->deck_size + 1;
 	}
+	elsif ( $position < 0 ) {
+		$position = $self->deck_size + $position + 2;
+	}
 
 	splice @{$self->get_deck}, $position - 1, 0, $card;
 
@@ -869,16 +1088,51 @@ Just as in regular gambling, you can deal cards from other positions:
 
 For more information on false dealing see the L<SEE ALSO> section.
 
+If you're dealing cards to a pile you can actually state where you're dealing:
+
+	$deck->deal( $pile );
+
+You can still do a false deal to a pile:
+
+	$deck->deal( 'second', $pile );
+
+	# or
+
+	$deck->deal( $pile, 'second' );
+
+Dealing from an empty deck won't do anything, but a warning will be issued.
+
 =cut
 
 sub deal {
 	my $self = shift;
-	my $deal = shift || 'top';
 
-	# TODO: what if $deal is unknown?
+	if (not $self->size) {
+		warn "Tried to deal without cards.\n";
+		return $self;
+	}
+
+	my $deal        = 'top';
+	my $destination = undef;
+	while (my $param = shift) {
+		if ( ref( $param ) eq 'Games::Cards::ShuffleTrack' ) {
+			$destination = $param;
+		}
+		elsif ( exists $shortcuts->{$param} ) {
+			$deal = $param;
+		}
+	}
+
 	my $position = $shortcuts->{$deal};
 
-	return $self->remove( $position );
+	my $card = $self->remove( $position );
+
+	if ( defined $destination ) {
+		return $destination->put( $card );
+	}
+	else {
+		return $card;
+	}
 }
 
 =head3 remove
@@ -890,6 +1144,7 @@ Removes a card from the deck.
 
 =cut
 
+# TODO: allow removal of several cards (do note that positions change as cards are removed)
 sub remove {
 	my $self     = shift;
 	my $position = shift;
@@ -964,9 +1219,97 @@ sub take_random {
 	return $self->remove( _rand( $lower_limit, $upper_limit ) );
 }
 
+=head3 dribble
+
+Dribble the cards (usually to select one, which could either be the last one to fall or the one that would be next).
+
+	# dribble cards onto $pile
+	$deck->dribble( $pile );
+
+	# same thing, but declaring $pile
+    $pile = $deck->dribble;
+
+    # dribble to position 10 (in a 52 card deck, 42 cards would fall)
+    $deck->dribble( 10 );
+
+    # dribble 10 cards
+    $deck->dribble( -10 );
+
+    # dribble to position between 10 and 20
+    $deck->dribble( 10, 20 );
+
+    # dribble to position between 10th from the top and 10th from the bottom
+    $deck->dribble( 10, -10 );
+
+=cut
+
+# TODO: what happens if you're dribbling and have no cards?
+sub dribble {
+	my $self   = shift;
+	my $params = _parse_params( @_ );
+
+	my $has_destination = @{$params->{'places'}};
+	my $destination = $has_destination ?
+						$params->{'places'}[0] :
+						Games::Cards::ShuffleTrack->new( 'empty' );
+
+	my ($lower_limit, $upper_limit) = $self->_fix_limits( @{$params->{'numbers'}} );
+
+	if ( defined $lower_limit ) {
+		$lower_limit = $self->size - $lower_limit;
+		$upper_limit = defined $upper_limit ?
+						$self->size - $upper_limit :
+						$lower_limit;
+	}
+	else {
+		# FIXME: this means we can't dribble if we have just 5 cards
+		$lower_limit = 5; # minimum amount of cards to hold back
+		$upper_limit = $self->size - 5; # minimum amount of cards to dribble
+	}
+
+	$self->turn;
+	my $transfer = $self->cut_to( $lower_limit, $upper_limit );
+	$transfer->turn;
+	$transfer->move_to( $destination );
+
+	return $has_destination ? $self : $destination;
+}
 
 # subroutines
 
+# TODO: fix limits in other methods (just being used in dribble)
+sub _fix_limits {
+	my $self = shift;
+	my @limits;
+
+	while ( my $limit = shift ) {
+		push @limits, $limit < 0 ? $self->size + $limit : $limit;
+	}
+
+	return @limits;
+}
+
+# TODO: use this for every method (just being used in dribble)
+sub _parse_params {
+	my $params = {
+		numbers => [],
+		places  => [],
+		options => [],
+	};
+	while (my $param = shift) {
+		if (looks_like_number($param)) {
+			push @{$params->{'numbers'}}, $param;
+		}
+		elsif (ref $param eq 'Games::Cards::ShuffleTrack') {
+			push @{$params->{'places'}}, $param;
+		}
+		else {
+			push @{$params->{'options'}}, $param;
+		}
+	}
+	return $params;
+}
+
 sub _set_deck {
 	my $self = shift;
 	return $self->{'deck'} = [@_];
@@ -1001,6 +1344,59 @@ sub _is_shortcut {
 }
 
 
+=head1 GLOSSARY
+
+The following is not a comprehensive list of gambling terms; it is simply a list of some terms used somewhere in this module that may be useful.
+
+The text has been taken verbatim from The Expert at the Card Table.
+
+=over 4
+
+=item * Stock
+
+That portion of the deck that contains certain cards, placed in some particular order for dealing; or certain
+desirable cards placed at top or bottom of the deck.
+
+=item * Run
+
+To draw off one card at a time during the process of the hand shuffle. There is little or no difficulty in acquiring
+perfect ability to run the whole deck through in this manner with the utmost rapidity. The left thumb presses
+lightly on the top card, the right hand alone making the movement necessary to shuffle.
+
+=item * Break
+
+A space or division held in the deck. While shuffling it is held at the end by the right thumb. It is formed under
+the in-jog when about to under cut for the shuffle, by pushing the in-jog card slightly upwards with the right
+thumb, making a space of from an eighth to a quarter of an inch wide, and holding the space, by squeezing the
+ends of the packet to be drawn out, between the thumb and second and third fingers. The use of the break
+during a shuffle makes it possible to throw any number of cards that are immediately above it, in one packet
+into the left hand, without disarranging their order. The break is used when not shuffling, to locate any particular
+card or position, and is infinitely superior to the common method of inserting the little finger. A break can be
+held firmly by a finger or thumb of either hand, and entirely concealed by the other fingers of the same hand. It
+is also the principal aid in the blind riffles and cuts.
+
+=item * Throw
+
+To pass from the right hand to the left, during a shuffle, a certain number of cards in one packet, thereby
+retaining their order. A throw may be required at the beginning, during the process, or at the end of a shuffle;
+and the packet to be thrown may be located by the jog, or break, or by both.
+
+=item * Top Card
+
+The card on top of packet held in the left hand, or the original top card of the full deck, which about to be
+shuffled.
+
+=item * Riffle
+
+The modern method of shuffling on the table by springing, the ends of two packets into each other.
+
+=item * Crimp
+
+To bend one or a number of cards, so that they may be distinguished or located. 
+
+=back
+
+
 =head1 AUTHOR
 
 Jose Castro, C<< <cog at cpan.org> >>
@@ -1036,13 +1432,13 @@ L<http://search.cpan.org/dist/Games-Cards-ShuffleTrack/>
 
 =head1 SEE ALSO
 
-Recommended reading:
+The following is an extremely small list of recommended books:
 
 =over 4
 
-=item * The Expert at the Card Table: The Classic Treatise on Card Manipulation, by S. W. Erdnase
+=item * The Expert at the Card Table, by S. W. Erdnase
 
-=item * The Annotated Erdnase, by Darwin Ortiz
+=item * Card College, by Roberto Giobbi
 
 =back
 
